/**
 * JsonCommentStripper: Eats up all comments a JSON file brings along.
 * <p>
 * Example:
 * <p>
 * <code>groovy JsonCommentStripper.groovy Sample.json > Sample-Stripped.json</code>
 * 
 * @author JCMeyer
 */
public class JsonCommentStripper {

	protected static final char space = ' '
	
	/**
	 * Eats up all comments a JSON string brings along. Won't change line or column numbers.
	 * <p>
	 * Some nerdy background information:
	 * <a href="https://plus.google.com/+DouglasCrockfordEsq/posts/RK8qyGVaGSr">https://plus.google.com/+DouglasCrockfordEsq/posts/RK8qyGVaGSr</a>
	 * 
	 * @param input the JSON string to strip
	 * @return the stripped JSON string
	 */
	public static CharSequence strip(CharSequence input) {

		CharSequence output = ''
		def state = 'code'
		
		for (int i = 0; i < input.length(); i++) {
			
			char c = input.charAt(i)
			
			if (state == 'code') {
				if (c == '"') {
					state = 'string'
					output += c;
				}
				else if (c == '/') {
					state = 'b_cmnt'
					output += space
				}
				else {
					output += c;
				}
			}
			else if (state == 'b_cmnt') {
				if (c == '/')
				{
					state = 'sl_cmnt'
					output += space
				}
				else if (c == '*') {
					state = 'ml_cmnt'
					output += space
				}
				else {
					state = 'code'
					def arr = output.toCharArray()
					arr[arr.size() - 1] = '/'
					output = '' + arr + c
				}
			}
			else if (state == 'sl_cmnt') {
				if (c == '\n') {
					state = 'code'
					output += c
				}
				else {
					output += space
				}
			}
			else if (state == 'ml_cmnt') {
				if (c == '*') {
					state = 'e_ml_cmnt'
					output += space
				}
				else if (c == '\n') {
					output += c
				}
				else {
					output += space
				}
			}
			else if (state == 'e_ml_cmnt') {
				if (c == '/') {
					state = 'code'
				}
				else {
					state = 'ml_cmnt'
				}
				output += space
			}
			else if (state == 'string') {
				if (c == '"') {
					state = 'code'
				}
				else if (c == '\\') {
					state = 'escaped'
				}
				output += c;
			}
			else if (state == 'escaped') {
				state = 'code'
				output += c;
			}
			
		}
		return output
	}

	public static void main(String[] args) {
		CharSequence input = new File(args[0]).getText(args.size() > 1 ? args[1] : 'UTF-8')
		print strip(input)
	}

}

